<?php

/*******************************************************************************
 *        MD5 Format Functions
 ******************************************************************************/

function _uc_paykeeper_verify_md5 (&$order_id, &$sum, &$payment_id){
  $order_id = check_plain($_POST['clientid']);
  $sum = sprintf("%.2lf", (float)$_POST['sum']);
  $payment_id = check_plain($_POST['id']);
  $secret = variable_get('uc_paykeeper_secret_seed');

  if (strcmp($_POST['key'], md5($payment_id.$sum.$order_id.$secret))!==0)
  {
# hash mismatch, possible error or intrusion attempt
    watchdog('uc_paykeeper', 'Incorrect incoming message hash', NULL, WATCHDOG_ERROR);
    echo "Hash mismatch";
    return FALSE;
  }

  return TRUE;
}

function _uc_paykeeper_answer_md5 ($payment_id){
  $secret = variable_get('uc_paykeeper_secret_seed');
  echo "OK ".md5($payment_id.$secret);

  return TRUE;
}

/*******************************************************************************
 *        RSA Format Functions
 ******************************************************************************/

function _uc_paykeeper_verify_rsa (&$order_id, &$sum, &$payment_id) {
  $pubkey_pem = variable_get('uc_paykeeper_pubkey');
  $pubkey = openssl_pkey_get_public($pubkey_pem);
  if ($pubkey == FALSE)
  {
    watchdog('uc_paykeeper', 'Invalid public key', NULL, WATCHDOG_ERROR);
    return FALSE;
  }

  $inmsg = file_get_contents("php://input");
  $encmsg = base64_decode($inmsg);
  $decrmsg = "";

  if (!openssl_public_decrypt($encmsg, $decrmsg, $pubkey))
  {
    watchdog('uc_paykeeper', 'Decryption failed', NULL, WATCHDOG_ERROR);
    return FALSE;
  }

  $params = null;
  parse_str($decrmsg, $params);

  $order_id = check_plain($params['clientid']);
  $sum = (float)$params['sum'];
  $payment_id = check_plain($params['id']);

  return TRUE;
}

function _uc_paykeeper_answer_rsa ($payment_id) {
  $pubkey_pem = variable_get('uc_paykeeper_pubkey');
  $pubkey = openssl_pkey_get_public($pubkey_pem);
  if ($pubkey == FALSE)
  {
    watchdog('uc_paykeeper', 'Invalid public key', NULL, WATCHDOG_ERROR);
    return FALSE;
  }

  $answer = "OK $payment_id";
  $encmsg = "";
  if (!openssl_public_encrypt($answer, $encmsg, $pubkey))
  {
    watchdog('uc_paykeeper', 'Answer encryption failed', NULL, WATCHDOG_ERROR);
    return FALSE; 
  }

  $tosend = base64_encode($encmsg);

  echo $tosend;
  return TRUE;
}


/*******************************************************************************
 *        Ubercart Related Funcitons 
 ******************************************************************************/

/*
 * Change order status and add payment 
 */

function _uc_paykeeper_process_order ($order_id, $sum, $payment_id){
  $order = uc_order_load($order_id);

  $targs = array(
      '@order'   => $order_id,
      '@paymentid' => $payment_id,
      '@sum'  => uc_currency_format($sum),
  );

  if (abs(((float)$order->order_total)*100 - ((float)$sum) * 100) > 2)
  {
    watchdog('uc_paykeeper', "Payed sum @sum differs from order @orderid total. Payment ID @paymentid", $targs, WATCHDOG_WARNING);
    echo "Sum mismatch";
    return FALSE;
  }

  $comment = t('Received payment of @sum for order #@order from PayKeeper. Payment id is @paymentid', $targs);

  $order->data['paykeeper_payment_id'] =  $payment_id;
  uc_order_save($order);

  uc_payment_enter($order->order_id, 'paykeeper', $sum, 0, array("payment_id"=>$payment_id), $comment);

  uc_cart_complete_sale($order, FALSE);
  uc_order_comment_save($order->order_id, 0, t('Payment has been accepted'), 'admin', 1, TRUE);

  return TRUE;
}

?>
